/*
 * jfdct_bin_c_trac.c
 *
 * This is the original binDCt Version C, according to Trac's paper.
 *
 */


/*
 ************************************************
 *
 * $Log$
 *
 ************************************************
 */

/*
**********************************************************************
*
* Modification History:
* Date       Programmer   Description
* --------   ----------   --------------------------------------------
*
***********************************************************************
*/

#define JPEG_INTERNALS
#include "jinclude.h"
#include "jpeglib.h"
#include "jdct.h"		/* Private declarations for DCT subsystem */

#ifdef DCT_BIN_L1_SUPPORTED


/*
 * This module is specialized to the case DCTSIZE = 8.
 */

#if DCTSIZE != 8
  Sorry, this code only copes with 8x8 DCTs. /* deliberate syntax err */
#endif

#if BITS_IN_JSAMPLE == 8
#define CONST_BITS  13
#define PASS1_BITS  2
#else
#define CONST_BITS  13
#define PASS1_BITS  1		/* lose a little precision to avoid overflow */
#endif

/* Some C compilers fail to reduce "FIX(constant)" at compile time, thus
 * causing a lot of useless floating-point operations at run time.
 * To get around this we use the following pre-calculated constants.
 * If you change CONST_BITS you may want to add appropriate values.
 * (With a reasonable C compiler, you can just rely on the FIX() macro...)
 */

/* Multiply an INT32 variable by an INT32 constant to yield an INT32 result.
 * For 8-bit samples with the recommended scaling, all the variable
 * and constant values involved are no more than 16 bits wide, so a
 * 16x16->32 bit multiply can be used instead of a full 32x32 multiply.
 * For 12-bit samples, a full 32-bit multiplication will be needed.
 */

#if BITS_IN_JSAMPLE == 8
#define MULTIPLY(var,const)  MULTIPLY16C16(var,const)
#else
#define MULTIPLY(var,const)  ((var) * (const))
#endif


/*
 * Perform the forward DCT on one block of samples.
 */

GLOBAL(void)
jpeg_fdct_bin_l1 (DCTELEM * data)
{
  INT32 tmp0, tmp1, tmp2, tmp3, tmp4, tmp5, tmp6, tmp7;
  INT32 tmp10, tmp11, tmp12, tmp13;
  INT32 z1, z2, z3, z4, z5;
  DCTELEM *dataptr;
  int ctr;
  SHIFT_TEMPS


  /* Pass 1: process rows. */
  /* Note results are scaled up by sqrt(8) compared to a true DCT; */
  /* furthermore, we scale the results by 2**PASS1_BITS. */

  dataptr = data;
  for (ctr = DCTSIZE-1; ctr >= 0; ctr--) {
    tmp0 = dataptr[0] + dataptr[7];
    tmp7 = dataptr[0] - dataptr[7];
    tmp1 = dataptr[1] + dataptr[6];
    tmp6 = dataptr[1] - dataptr[6];
    tmp2 = dataptr[2] + dataptr[5];
    tmp5 = dataptr[2] - dataptr[5];
    tmp3 = dataptr[3] + dataptr[4];
    tmp4 = dataptr[3] - dataptr[4];
    
    /* Even part */
    
    tmp10 = (tmp0 + tmp3) ;	/* phase 2 */
    tmp13 = tmp0 - tmp3;
    tmp11 = (tmp1 + tmp2) ;
    tmp12 = tmp1 - tmp2;
    
    dataptr[0] = (tmp10 + tmp11); /* phase 3 */
    dataptr[4]=tmp10-tmp11;




    dataptr[6] = tmp12 - (((tmp13 << 1) + tmp13 + 4) >> 3);
    dataptr[2] = tmp13 + (((dataptr[6] << 1) + dataptr[6] + 4) >> 3);
    
    /* Odd part */

    tmp10 = tmp4;	/* phase 2 */
    tmp13 = tmp7;

/* pi/4 = -7/16u 11/16d -3/8u*/
    tmp12 = tmp6 + (((tmp5<<1) + tmp5 + 4) >> 3);
    tmp11 = (((tmp12<<2) + tmp12 + 4) >> 3) -  tmp5;

    tmp1 = (tmp10+tmp11) ;
    tmp4 = (tmp12+tmp13) ;
    tmp2 = tmp10-tmp11;
    tmp3 = -tmp12+tmp13;

    /* 7pi/16 = -3/16u 3/16d */
    dataptr[1] = tmp4;
    dataptr[7] = tmp1-((tmp4+4) >> 3);

    /* 3pi/16 = 7/8u -1/2d */

    dataptr[5] = tmp2 + (((tmp3<<2) + (tmp3<<1) + tmp3 + 4) >> 3);
    dataptr[3] = tmp3 - ((dataptr[5] + 1) >> 1);
    
    dataptr += DCTSIZE;		/* advance pointer to next row */
  }

  /* Pass 2: process columns.
   * We remove the PASS1_BITS scaling, but leave the results scaled up
   * by an overall factor of 8.
   */

  dataptr = data;
  for (ctr = DCTSIZE-1; ctr >= 0; ctr--) {
    tmp0 = (dataptr[DCTSIZE*0] + dataptr[DCTSIZE*7]);
    tmp7 = (dataptr[DCTSIZE*0] - dataptr[DCTSIZE*7]);
    tmp1 = (dataptr[DCTSIZE*1] + dataptr[DCTSIZE*6]);
    tmp6 = (dataptr[DCTSIZE*1] - dataptr[DCTSIZE*6]);
    tmp2 = (dataptr[DCTSIZE*2] + dataptr[DCTSIZE*5]);
    tmp5 = (dataptr[DCTSIZE*2] - dataptr[DCTSIZE*5]);
    tmp3 = (dataptr[DCTSIZE*3] + dataptr[DCTSIZE*4]);
    tmp4 = (dataptr[DCTSIZE*3] - dataptr[DCTSIZE*4]);
    
    /* Even part */
    
    tmp10 = (tmp0 + tmp3);	/* phase 2 */
    tmp13 = tmp0 - tmp3;
    tmp11 = (tmp1 + tmp2);
    tmp12 = tmp1 - tmp2;
    
    dataptr[DCTSIZE*0] = (tmp10 + tmp11); /* phase 3 */
    dataptr[DCTSIZE*4]=tmp10-tmp11;
    dataptr[DCTSIZE*6] = tmp12 - (((tmp13 << 1) + tmp13 + 4) >> 3);
    dataptr[DCTSIZE*2] = tmp13 + (((dataptr[DCTSIZE*6] << 1) + dataptr[DCTSIZE*6] + 4) >> 3);

    /* Odd part */

    tmp10 = tmp4;	/* phase 2 */
    tmp13 = tmp7;

/* pi/4 = -7/16u 11/16d -3/8u*/
    tmp12 = tmp6 + (((tmp5<<1) + tmp5 + 4) >> 3);
    tmp11 = (((tmp12<<2) + tmp12 + 4) >> 3) -  tmp5;

    tmp1 = (tmp10+tmp11) ;
    tmp4 = (tmp12+tmp13) ;
    tmp2 = tmp10-tmp11;
    tmp3 = -tmp12+tmp13;

    /* 7pi/16 = -3/16u 3/16d */
    dataptr[DCTSIZE*1] = tmp4;
    dataptr[DCTSIZE*7] = tmp1-((tmp4+4) >> 3);
    

    /* 3pi/16 = 7/8u -1/2d */

    dataptr[DCTSIZE*5] = tmp2 + (((tmp3<<2) + (tmp3<<1) + tmp3 + 4) >> 3);
    dataptr[DCTSIZE*3] = tmp3 - ((dataptr[DCTSIZE*5] + 1) >> 1);

    dataptr++;			/* advance pointer to next column */
  }
}

#endif /* DCT_BIN_L1_SUPPORTED */
