/*
 * jidct_bin_l1.c
 *
 * This is the Trac's original binDCT version C.
 *
 */
#define JPEG_INTERNALS
#include "jinclude.h"
#include "jpeglib.h"
#include "jdct.h"		/* Private declarations for DCT subsystem */

#ifdef DCT_BIN_L1_SUPPORTED


/*
 * This module is specialized to the case DCTSIZE = 8.
 */

#if DCTSIZE != 8
  Sorry, this code only copes with 8x8 DCTs. /* deliberate syntax err */
#endif


#if BITS_IN_JSAMPLE == 8
#define CONST_BITS  13
#define PASS1_BITS  2
#else
#define CONST_BITS  13
#define PASS1_BITS  1		/* lose a little precision to avoid overflow */
#endif

/* Multiply an INT32 variable by an INT32 constant to yield an INT32 result.
 * For 8-bit samples with the recommended scaling, all the variable
 * and constant values involved are no more than 16 bits wide, so a
 * 16x16->32 bit multiply can be used instead of a full 32x32 multiply.
 * For 12-bit samples, a full 32-bit multiplication will be needed.
 */

#if BITS_IN_JSAMPLE == 8
#define MULTIPLY(var,const)  MULTIPLY16C16(var,const)
#else
#define MULTIPLY(var,const)  ((var) * (const))
#endif


/* Dequantize a coefficient by multiplying it by the multiplier-table
 * entry; produce an int result.  In this module, both inputs and result
 * are 16 bits or less, so either int or short multiply will work.
 */

#define DEQUANTIZE(coef,quantval)  (((ISLOW_MULT_TYPE) (coef)) * (quantval))


/*
 * Perform dequantization and inverse DCT on one block of coefficients.
 */

GLOBAL(void)
jpeg_idct_bin_l1 (j_decompress_ptr cinfo, jpeg_component_info * compptr,
		 JCOEFPTR coef_block,
		 JSAMPARRAY output_buf, JDIMENSION output_col)
{
  INT32 tmp0, tmp1, tmp2, tmp3,tmp4,tmp5,tmp6,tmp7;
  INT32 tmp10, tmp11, tmp12, tmp13;
  INT32 z0,z1, z2, z3, z4,z10,z11,z12,z13;
  JCOEFPTR inptr;
  ISLOW_MULT_TYPE * quantptr;
  int * wsptr;
  JSAMPROW outptr;
  JSAMPLE *range_limit = IDCT_range_limit(cinfo);
  int ctr;
  int workspace[DCTSIZE2];	/* buffers data between passes */
  SHIFT_TEMPS

  /* Pass 1: process columns from input, store into work array. */
  /* Note results are scaled up by sqrt(8) compared to a true IDCT; */
  /* furthermore, we scale the results by 2**PASS1_BITS. */

  inptr = coef_block;
  quantptr = (ISLOW_MULT_TYPE *) compptr->dct_table;
  wsptr = workspace;
  for (ctr = DCTSIZE; ctr > 0; ctr--) {
    /* Due to quantization, we will usually find that many of the input
     * coefficients are zero, especially the AC terms.  We can exploit this
     * by short-circuiting the IDCT calculation for any column in which all
     * the AC terms are zero.  In that case each output is equal to the
     * DC coefficient (with scale factor as needed).
     * With typical images and quantization tables, half or more of the
     * column DCT calculations can be simplified this way.
     */
    if (inptr[DCTSIZE*1] == 0 && inptr[DCTSIZE*2] == 0 &&
	inptr[DCTSIZE*3] == 0 && inptr[DCTSIZE*4] == 0 &&
	inptr[DCTSIZE*5] == 0 && inptr[DCTSIZE*6] == 0 &&
	inptr[DCTSIZE*7] == 0) {
      int dcval = DEQUANTIZE(inptr[DCTSIZE*0], quantptr[DCTSIZE*0]) >> 1;
      
      wsptr[DCTSIZE*0] = dcval;
      wsptr[DCTSIZE*1] = dcval;
      wsptr[DCTSIZE*2] = dcval;
      wsptr[DCTSIZE*3] = dcval;
      wsptr[DCTSIZE*4] = dcval;
      wsptr[DCTSIZE*5] = dcval;
      wsptr[DCTSIZE*6] = dcval;
      wsptr[DCTSIZE*7] = dcval;
      
      inptr++;			
      quantptr++;
      wsptr++;
      continue;
    }
    
    tmp0 = DEQUANTIZE(inptr[DCTSIZE*0], quantptr[DCTSIZE*0]);
    tmp1 = DEQUANTIZE(inptr[DCTSIZE*2], quantptr[DCTSIZE*2]);
    tmp2 = DEQUANTIZE(inptr[DCTSIZE*4], quantptr[DCTSIZE*4]);
    tmp3 = DEQUANTIZE(inptr[DCTSIZE*6], quantptr[DCTSIZE*6]);
    tmp4 = DEQUANTIZE(inptr[DCTSIZE*1], quantptr[DCTSIZE*1]);
    tmp5 = DEQUANTIZE(inptr[DCTSIZE*3], quantptr[DCTSIZE*3]);
    tmp6 = DEQUANTIZE(inptr[DCTSIZE*5], quantptr[DCTSIZE*5]);
    tmp7 = DEQUANTIZE(inptr[DCTSIZE*7], quantptr[DCTSIZE*7]);

	/* 7pi/16 = -3/16d 3/16u */

    tmp10 = tmp4;
    tmp11 = tmp7 +((tmp4 + 4)>>3);


	/* 3pi/16 = 1/2d -7/8u */

    tmp13 = tmp5 + ((tmp6 + 1) >> 1);
    tmp12 = tmp6 - (((tmp13 << 2) + (tmp13 << 1) + tmp13 + 4) >> 3);

     z11=(tmp0-tmp2)>>1;
     z10=(tmp0+tmp2)>>1;

    /* 3pi/8 = -3/8d 3/8u */
    z13 = tmp1 - (((tmp3 << 1) + tmp3 + 4) >> 3);
    z12 = tmp3 + (((z13 << 1) + z13 + 4) >> 3);

    tmp4 = tmp11 + tmp12;
    tmp5 = tmp11 - tmp12;
    tmp7 = tmp10 + tmp13;
    tmp6 = tmp10 - tmp13;

	/* pi/4 = -3/8u -11/16d 7/16u */

    tmp5 = (((tmp6<<2) + tmp6 + 4) >> 3) - tmp5;
    tmp6 = tmp6 - (((tmp5<<1) + tmp5 + 4) >> 3);

    tmp0 = z10  + z13;
    tmp1 = z11 + z12;
    tmp2 = z11 - z12;
    tmp3 = z10 - z13;



    z10=tmp0 + tmp7;
    z11=tmp0 - tmp7;
    wsptr[DCTSIZE*0] = ((z10)) ;
    wsptr[DCTSIZE*7] = ((z11)) ;
    z10=tmp1 + tmp6;
    z11=tmp1 - tmp6;
    wsptr[DCTSIZE*1] = ((z10)) ;
    wsptr[DCTSIZE*6] = ((z11)) ;
    z10=tmp2 + tmp5;
    z11=tmp2 - tmp5;
    wsptr[DCTSIZE*2] = ((z10)) ;
    wsptr[DCTSIZE*5] = ((z11)) ;
    z10=tmp3 + tmp4;
    z11=tmp3 - tmp4;
    wsptr[DCTSIZE*3] = ((z10)) ;
    wsptr[DCTSIZE*4] = ((z11)) ;

    
    inptr++;			/* advance pointers to next column */
    quantptr++;
    wsptr++;
  }
  
  /* Pass 2: process rows from work array, store into output array. */
  /* Note that we must descale the results by a factor of 8 == 2**3, */
  /* and also undo the PASS1_BITS scaling. */

  wsptr = workspace;
  for (ctr = 0; ctr < DCTSIZE; ctr++) {
    outptr = output_buf[ctr] + output_col;
    /* Rows of zeroes can be exploited in the same way as we did with columns.
     * However, the column calculation has created many nonzero AC terms, so
     * the simplification applies less often (typically 5% to 10% of the time).
     * On machines with very fast multiplication, it's possible that the
     * test takes more time than it's worth.  In that case this section
     * may be commented out.
     */
    
#ifndef NO_ZERO_ROW_TEST
    if (wsptr[1] == 0 && wsptr[2] == 0 && wsptr[3] == 0 && wsptr[4] == 0 &&
	wsptr[5] == 0 && wsptr[6] == 0 && wsptr[7] == 0) {
      JSAMPLE dcval = range_limit[(int) DESCALE((INT32) wsptr[0], 2)
				  & RANGE_MASK];
      
      outptr[0] = dcval;
      outptr[1] = dcval;
      outptr[2] = dcval;
      outptr[3] = dcval;
      outptr[4] = dcval;
      outptr[5] = dcval;
      outptr[6] = dcval;
      outptr[7] = dcval;

      wsptr += DCTSIZE;		
      continue;
    }
#endif
    
    /* Even part: reverse the even part of the forward DCT. */
    /* The rotator is sqrt(2)*c(-6). */

    /* Even part */
    tmp0 = (INT32) wsptr[0];
    tmp1 = (INT32) wsptr[2];
    tmp2 = (INT32) wsptr[4];
    tmp3 = (INT32) wsptr[6];
    tmp4 = (INT32) wsptr[1];
    tmp5 = (INT32) wsptr[3];
    tmp6 = (INT32) wsptr[5];
    tmp7 = (INT32) wsptr[7];

	/* 7pi/16 = -3/16d 3/16u */

    tmp10 = tmp4;
    tmp11 = tmp7 +((tmp4 + 4)>>3);

	/* 3pi/16 = 1/2d -7/8u */

    tmp13 = tmp5 + ((tmp6 + 1) >> 1);
    tmp12 = tmp6 - (((tmp13 << 2) + (tmp13 << 1) + tmp13 + 4) >> 3);

     z11=(tmp0-tmp2)>>1;
     z10=(tmp0+tmp2)>>1;

    /* 3pi/8 = -3/8d 3/8u */
    z13 = tmp1 - (((tmp3 << 1) + tmp3 + 4) >> 3);
    z12 = tmp3 + (((z13 << 1) + z13 + 4) >> 3);

    tmp4 = tmp11 + tmp12;
    tmp5 = tmp11 - tmp12;
    tmp7 = tmp10 + tmp13;
    tmp6 = tmp10 - tmp13;

	/* pi/4 = -3/8u -11/16d 7/16u */

    tmp5 = (((tmp6<<2) + tmp6 + 4) >> 3) - tmp5;
    tmp6 = tmp6 - (((tmp5<<1) + tmp5 + 4) >> 3);

    tmp0 = z10 + z13;
    tmp1 = z11 + z12;
    tmp2 = z11 - z12;
    tmp3 = z10 - z13;


    /* Final output stage: scale down by a factor of 8 and range-limit */
    z10=(tmp0 + tmp7);
    z11=(tmp0 - tmp7);
    outptr[0] = range_limit[(int)DESCALE(z10,1) & RANGE_MASK];
    outptr[7] =  range_limit[(int)DESCALE(z11,1) & RANGE_MASK];
    z10=(tmp1 + tmp6);
    z11=(tmp1 - tmp6);
    outptr[1] = range_limit[(int)DESCALE(z10,1) & RANGE_MASK];
    outptr[6] = range_limit[(int)DESCALE(z11,1) & RANGE_MASK];
    z10=(tmp2 + tmp5);
    z11=(tmp2 - tmp5);

    outptr[2] = range_limit[(int)DESCALE(z10,1) & RANGE_MASK];
    outptr[5] = range_limit[(int)DESCALE(z11,1) & RANGE_MASK];
    z10=(tmp3 + tmp4);
    z11=(tmp3 - tmp4);
    outptr[3] = range_limit[(int)DESCALE(z10,1) & RANGE_MASK];
    outptr[4] = range_limit[(int)DESCALE(z11,1) & RANGE_MASK];
    
    
    wsptr += DCTSIZE;		/* advance pointer to next row */
  }
}

#endif /* DCT_BIN_L1_SUPPORTED */
